IMPLEMENTATION MODULE Tables ;

FROM SysStorage IMPORT ALLOCATE ;
FROM Debug IMPORT Halt ;
FROM StdIO IMPORT Write ;
FROM StrIO IMPORT WriteString, WriteLn ;
FROM NumberIO IMPORT ReadCard, WriteCard ;
FROM StrLib IMPORT StrLen ;
FROM ASCII IMPORT nul ;


CONST
   TableLength = 6000 ;
   LineWidth   = 256 ;

TYPE
   Table   = POINTER TO Word ;
   Word    = RECORD
                key  : CARDINAL ;   (* Table Index *)
                left,
                right: Table ;
                val  : CARDINAL ;
             END ;


   Relation= (less, equal, greater) ;

VAR
   asc    : ARRAY [0..TableLength+LineWidth] OF CHAR ;
   ascinx : CARDINAL ;


(* %%%FORWARD%%%
PROCEDURE PrintItem (p: Table) ; FORWARD ;
PROCEDURE Rel (k: CARDINAL) : Relation ; FORWARD ;
PROCEDURE Search (t: Table; VAR id, father: Table) : Relation ; FORWARD ;
PROCEDURE TraverseTree (p: Table) ; FORWARD ;
PROCEDURE PrintItem (p: Table) ; FORWARD ;
   %%%FORWARD%%% *)


PROCEDURE InitTable () : Table ;
VAR
   t: Table ;
BEGIN
   NEW( t ) ;
   t^.left := NIL ;
   RETURN( t )
END InitTable ;


(*
   Compare asc[ascinx] with asc[k]
*)

PROCEDURE Rel (k: CARDINAL) : Relation ;
VAR
   i: CARDINAL ;
   r: Relation ;
   x, y: CHAR ;
BEGIN
   i := 0 ;
   x := asc[ascinx] ;
   y := asc[k] ;
   r := equal ;
   REPEAT
      IF x<y
      THEN
         r := less
      ELSIF x>y
      THEN
         r := greater
      ELSE
         INC(i) ;
         x := asc[ascinx+i] ;
         INC(k) ;
         y := asc[k] ;
         IF (x=nul) AND (y#nul)
         THEN
            r := less
         ELSIF (x#nul) AND (y=nul)
         THEN
            r := greater
         END
      END
   UNTIL (r#equal) OR (x=nul) OR (y=nul) ;
   RETURN(r)
END Rel ;


(*
   Search table, t,with name equal to a. id is set to the node if
   an entry is found, father is set to the node above id.
*)

PROCEDURE Search (t: Table; VAR id, father: Table) : Relation ;
VAR
   r: Relation ;
BEGIN
   father := t ;
   id := t^.left ;
   IF id=NIL
   THEN
      r := less
   ELSE
      REPEAT
         r := Rel( id^.key ) ;
         IF r=less
         THEN
            father := id ;
            id := id^.left
         ELSIF r=greater
         THEN
            father := id ;
            id := id^.right
         END
      UNTIL (id=NIL) OR (r=equal)
   END ;
   RETURN( r )
END Search ;


PROCEDURE Modify (t: Table; n: CARDINAL) : BOOLEAN ;
VAR
   id, f: Table ;
   r    : Relation ;
BEGIN
   r := Search(t, id, f) ;
   IF id=NIL
   THEN
      RETURN( FALSE )
   ELSE
      id^.val := n ;
      RETURN( TRUE )
   END
END Modify ;


PROCEDURE Exist (t: Table; a: ARRAY OF CHAR) : BOOLEAN ;
VAR
   f, c  : Table ;
   i,
   higha : CARDINAL ;
   r     : Relation ;
BEGIN
   (* Pseudo MACRO - place a[] into asc[] *)
   i := 0 ;
   higha := StrLen(a) ;
   WHILE i<higha DO
      asc[ascinx+i] := a[i] ;
      INC( i )
   END ;
   asc[ascinx+i] := nul ;
   (* END of MACRO *)

   r := Search(t, c, f) ;
   RETURN( c#NIL )
END Exist ;


PROCEDURE Record (t: Table; a: ARRAY OF CHAR; n: CARDINAL) ;
VAR
   r     : Relation ;
   f, c  : Table ;
   i,
   higha : CARDINAL ;
BEGIN
   IF ascinx>TableLength
   THEN
      (* Halt('Fatal Error --- Symbol Table Overflow') *)
      Halt('increase TableLength', __LINE__, __FILE__)
   ELSE
      (* Pseudo MACRO - place a[] into asc[] *)
      i := 0 ;
      higha := StrLen(a) ;
      WHILE i<higha DO
         asc[ascinx+i] := a[i] ;
         INC( i )
      END ;
      asc[ascinx+i] := nul ;
      (* END of MACRO *)

      r := Search(t, c, f) ;
      IF c=NIL
      THEN
         IF r=less
         THEN
            NEW( f^.left ) ;
            c := f^.left
         ELSIF r=greater
         THEN
            NEW( f^.right ) ;
            c := f^.right
         END ;
         WITH c^ DO
            right := NIL ;
            left := NIL ;
            key := ascinx ;
            val := n
         END ;
         INC( ascinx, higha+1 ) 
      ELSE
         c^.val := n
      END
   END
END Record ;


PROCEDURE Get (t: Table; a: ARRAY OF CHAR; VAR n: CARDINAL) : BOOLEAN ;
VAR
   i,
   higha: CARDINAL ;
   c, f : Table ;
   r    : Relation ;
BEGIN
   (* Pseudo MACRO - place a[] into asc[] *)
   i := 0 ;
   higha := StrLen(a) ;
   WHILE i<higha DO
      asc[ascinx+i] := a[i] ;
      INC( i )
   END ;
   asc[ascinx+i] := nul ;
   (* END of MACRO *)
   r := Search(t, c, f) ;
   IF c=NIL
   THEN
      RETURN( FALSE )
   ELSE
      n := c^.val ;
      RETURN( TRUE )
   END
END Get ;


PROCEDURE Tabulate (t: Table) ;
BEGIN
   WriteLn ;
   TraverseTree(t^.left)
END Tabulate ;


PROCEDURE TraverseTree (p: Table) ;
BEGIN
   IF p#NIL
   THEN
      TraverseTree(p^.left) ;
      PrintItem(p) ;
      TraverseTree(p^.right)
   END
END TraverseTree ;


PROCEDURE PrintItem (p: Table) ;
VAR
   ch : CHAR ;
   k  : CARDINAL ;
BEGIN
   WriteCard(p^.val, 6) ;
   WriteString('   ') ;
   k := p^.key ;
   REPEAT
      ch := asc[k] ;
      INC(k) ;
      Write(ch)
   UNTIL ch=nul ;
   WriteLn
END PrintItem ;


BEGIN
   ascinx := 0
END Tables.
