IMPLEMENTATION MODULE PortIO ;


(*
   Note that In and Out send a dummy byte to
        port 0x80 which (should be) an unused port.
        Maybe we never need this as In and Out are procedures?

        I borrowed/stole this idea from linux.
        (what a great system Linux is...)
*)


(*
   SlowDownIO - linux mechanism for slowing down port accesses. I suspect that
                a procedure call should be enough, but in these days of
                water cooled, rocket engineered pentiums you can never be sure.
*)

PROCEDURE SlowDownIO ;
BEGIN
   ASM
      outb %al,$0x80  # linux idea for slowing fast machines down
      outb %al,$0x80  # linux idea for slowing fast machines down
      outb %al,$0x80  # linux idea for slowing fast machines down
      outb %al,$0x80  # linux idea for slowing fast machines down
   END
END SlowDownIO ;


(*
   In8 - returns a BYTE from port, Port.
*)

PROCEDURE In8 (Port: CARDINAL) : BYTE ;
BEGIN
   SlowDownIO ;
   ASM
      pushl  %edx
      movl   8(%ebp), %edx  # Port number
      inb    %dx, %al
      popl   %edx

      outb %al,$0x80  # linux idea for slowing fast machines down
   END
END In8 ;


(*
   Out8 - sends a byte, Value, to port, Port.
*)

PROCEDURE Out8 (Port: CARDINAL; Value: BYTE) ;
BEGIN
   SlowDownIO ;
   ASM
      pushl  %edx
      movl   8(%ebp), %edx  # Port number
      movb   12(%ebp), %al  # Value
      outb   %al, %dx
      popl   %edx

      outb %al,$0x80  # linux idea for slowing fast machines down
   END
END Out8 ;


(*
   In16 - returns a WORD from port, Port.
          The top 16 bits are 0, the bottom 16 bits are assigned from
          the value of the port.
*)

PROCEDURE In16 (Port: CARDINAL) : WORD ;
BEGIN
   SlowDownIO ;
   ASM
      pushl  %edx
      movl   8(%ebp), %edx  # Port number
      movl   $0, %eax
      inw    %dx, %ax
      popl   %edx

      outb %al,$0x80  # linux idea for slowing fast machines down
   END
END In16 ;


(*
   Out16 - sends a 16 bit value, Value, to port, Port.
           Value is actually a 32 bit entity but the top
           16 bits are ignored.
*)

PROCEDURE Out16 (Port: CARDINAL; Value: WORD) ;
BEGIN
   SlowDownIO ;
   ASM
      pushl  %edx
      movl   8(%ebp), %edx  # Port number
      movb   12(%ebp), %ax  # Value
      outb   %ax, %dx
      popl   %edx

      outb %al,$0x80  # linux idea for slowing fast machines down
   END
END Out16 ;



(*
   InS8 - reads, n, bytes in from port, Port, to address, a.
*)

PROCEDURE InS8 (Port: CARDINAL; a: ADDRESS; n: CARDINAL) ;
BEGIN
   SlowDownIO ;
   ASM
      pushl %edi
      pushl %ecx
      pushl %edx
        
      movl 8(%ebp),%edx
      movl 12(%ebp),%edi
      movl 16(%ebp),%ecx

      cld ; rep ; insb  %dx,%edi

      popl %edx
      popl %ecx
      popl %edi

      # no need for slow down io here
   END
END InS8 ;


(*
   InS16 - reads, n, 16 bit words in from port, Port, to address, a.
*)

PROCEDURE InS16 (Port: CARDINAL; a: ADDRESS; n: CARDINAL) ;
BEGIN
   SlowDownIO ;
   ASM
      pushl %edi
      pushl %ecx
      pushl %edx
        
      movl 8(%ebp),%edx
      movl 12(%ebp),%edi
      movl 16(%ebp),%ecx

      cld ; rep ; insw  %dx,%edi

      popl %edx
      popl %ecx
      popl %edi

      # no need for slow down io here
   END
END InS16 ;


(*
   InS32 - reads, n, 32 bit words in from port, Port, to address, a.
*)

PROCEDURE InS32 (Port: CARDINAL; a: ADDRESS; n: CARDINAL) ;
BEGIN
   SlowDownIO ;
   ASM
      pushl %edi
      pushl %ecx
      pushl %edx
        
      movl 8(%ebp),%edx    # Port: CARDINAL
      movl 12(%ebp),%edi   # a   : Address
      movl 16(%ebp),%ecx   # n   : CARDINAL

      cld ; rep ; insl  %dx,%edi

      popl %edx
      popl %ecx
      popl %edi

      # no need for slow down io here
   END
END InS32 ;


(*
   OutS8 - writes, n, bytes to port, Port, from address, a.
*)

PROCEDURE OutS8 (Port: CARDINAL; a: ADDRESS; n: CARDINAL) ;
BEGIN
   SlowDownIO ;
   ASM
      pushl %esi
      pushl %ecx
      pushl %edx

      movl 8(%ebp),%edx
      movl 12(%ebp),%esi
      movl 16(%ebp),%ecx

      cld ; rep ; outsb  %dx,%edi

      popl %edx
      popl %ecx
      popl %esi

      # no need for slow down io here
   END
END OutS8 ;


(*
   OutS16 - writes, n, words to port, Port, from address, a.
*)

PROCEDURE OutS16 (Port: CARDINAL; a: ADDRESS; n: CARDINAL) ;
BEGIN
   SlowDownIO ;
   ASM
      pushl %esi
      pushl %ecx
      pushl %edx

      movl 8(%ebp),%edx
      movl 12(%ebp),%esi
      movl 16(%ebp),%ecx

      cld ; rep ; outsw  %dx,%edi

      popl %edx
      popl %ecx
      popl %esi

      # no need for slow down io here
   END
END OutS16 ;


(*
   OutS32 - writes, n, 32 bit words to port, Port, from address, a.
*)

PROCEDURE OutS32 (Port: CARDINAL; a: ADDRESS; n: CARDINAL) ;
BEGIN
   SlowDownIO ;
   ASM
      pushl %esi
      pushl %ecx
      pushl %edx

      movl 8(%ebp),%edx
      movl 12(%ebp),%esi
      movl 16(%ebp),%ecx

      cld ; rep ; outsl  %dx,%edi

      popl %edx
      popl %ecx
      popl %esi

      # no need for slow down io here
   END
END OutS32 ;


END PortIO.
