IMPLEMENTATION MODULE M2RTS ;


FROM StdIO IMPORT PushOutput, Write ;
FROM StrIO IMPORT WriteString, WriteLn ;
FROM NumberIO IMPORT CardToStr ;

IMPORT StdIO ;
IMPORT SysStorage ;

FROM libc IMPORT abort, exit ;
(* was FROM SYSTEM IMPORT ADDRESS, PROC ; *)
FROM ASCII IMPORT nl, nul ;
FROM SYSTEM IMPORT ADDRESS ;


CONST
   Max = 10 ;
VAR
   Ptr      : CARDINAL ;
   List     : ARRAY [0..Max] OF PROC ;
   ExitValue: INTEGER ;
   CallExit : BOOLEAN ;


(*
   Terminate - calls each installed termination procedure in turn.
*)

PROCEDURE Terminate ;
VAR
   i: CARDINAL ;
BEGIN
   i := 0 ;
   WHILE i<Ptr DO
      List[i] ;
      INC(i)
   END
END Terminate ;


(*
   HALT - terminate the current program calling creating a core dump.
          The procedure Terminate is called before the core dump is
          created.
*)

PROCEDURE HALT ;
BEGIN
   Terminate ;
   IF CallExit
   THEN
      exit(ExitValue)
   ELSE
      abort
   END
END HALT ;


(*
   ErrorMessage - emits an error message to the stderr
*)

PROCEDURE ErrorMessage (message: ARRAY OF CHAR; file: ARRAY OF CHAR; line: CARDINAL) ;
VAR
   LineNo: ARRAY [0..10] OF CHAR ;
BEGIN
   PushOutput(Write) ;
   WriteString(file) ; WriteString(':') ;
   CardToStr(line, 0, LineNo) ;
   WriteString(LineNo) ; WriteString(':') ;
   WriteString(message) ;
   LineNo[0] := nl ; LineNo[1] := nul ;
   WriteString(LineNo) ;
   HALT
END ErrorMessage ;


(*
   SubrangeAssignmentError - part of the runtime checking, called if a
                             subrange variable is just about to be assigned an illegal value.
*)

PROCEDURE SubrangeAssignmentError (file: ARRAY OF CHAR; line: CARDINAL) ;
BEGIN
   ErrorMessage('variable exceeds subrange', file, line)
END SubrangeAssignmentError ;


(*
   ArraySubscriptError -  part of the runtime checking, called if an
                          array indice is out of range.
*)

PROCEDURE ArraySubscriptError (file: ARRAY OF CHAR; line: CARDINAL) ;
BEGIN
   ErrorMessage('array index out of bounds', file, line)
END ArraySubscriptError ;


(*
   FunctionReturnError -  part of the runtime checking, called if a
                          function exits without a RETURN statement.
*)

PROCEDURE FunctionReturnError (file: ARRAY OF CHAR; line: CARDINAL) ;
BEGIN
   ErrorMessage('function is attempting to exit without a formal RETURN statement', file, line)
END FunctionReturnError ;


(*
   ExitOnHalt - if HALT is executed then call exit with the exit code, e.
*)

PROCEDURE ExitOnHalt (e: INTEGER) ;
BEGIN
   ExitValue := e ;
   CallExit := TRUE
END ExitOnHalt ;


(*
   InstallTerminationProcedure - installs a procedure, p, which will
                                 be called when the procedure Terminate
                                 is ionvoked.
*)

PROCEDURE InstallTerminationProcedure (p: PROC) ;
BEGIN
   List[Ptr] := p ;
   INC(Ptr)
END InstallTerminationProcedure ;


(*
   Init - initialize this module data structures.
*)

PROCEDURE Init ;
BEGIN
   Ptr := 0 ;
   ExitValue := 0 ;
   CallExit := FALSE   (* default by calling abort *)
END Init ;


BEGIN
   (*
      it is the job of M2RTS to call each all runtime initialization
      routines. We need a global initialization routine as the order
      it critical and we dare not let the statical analysis guess it..
   *)
   Init ;          (* M2RTS    initialization                      *)
   StdIO.Init ;    (* StdIO    initialization: use Scn.Write       *)
   SysStorage.Init (* Storage  initialize the heap                 *)
END M2RTS.
