IMPLEMENTATION MODULE SysStorage ;

(*
   Standard dynamic storage management

Storage management for dynamic variables. Calls to the
Modula-2 standard procedures NEW and DISPOSE are
translated into calls to ALLOCATE and DEALLOCATE. The
standard way to provide these two procedures is to
import them from this module 'Storage'.

Derived from the Lilith Modula-2 system developed by the
group of Prof. N. Wirth at ETH Zurich, Switzerland.
*)

IMPORT M2RTS ; (* this is only imported to tell the M2 startup analyser *)
               (* that Storage is dependant upon M2RTS                  *)

FROM SYSTEM IMPORT BYTE, ADR,
                   TurnInterrupts, OnOrOff ;

CONST
   MaxHeap = 100 * 01000H ;

VAR
   TopOfHeap: ADDRESS ;
   Used     : CARDINAL ;
   Left     : CARDINAL ;
   Store    : ARRAY [0..MaxHeap] OF BYTE ;


PROCEDURE ALLOCATE (VAR a: ADDRESS; Size: CARDINAL) ;
VAR
   ToOldState: OnOrOff ;
BEGIN
   ToOldState := TurnInterrupts(Off) ;
   IF Left<Size
   THEN
      HALT
   ELSE
      INC(Used, Size) ;
      a := ADDRESS(CARDINAL(TopOfHeap)-Used) ;
      DEC(Left, Size)
   END ;
   ToOldState := TurnInterrupts(ToOldState)
END ALLOCATE ;


PROCEDURE DEALLOCATE (a: ADDRESS; Size: CARDINAL) ;
BEGIN
END DEALLOCATE ;


(*
   Available - returns TRUE if, Size, bytes can be allocated.
*)

PROCEDURE Available (Size: CARDINAL) : BOOLEAN;
BEGIN
   RETURN( Size<=Left )
END Available ;


(*
   Init - initializes the heap.
*)

PROCEDURE Init ;
BEGIN
   Used := 0 ;
   Left := MaxHeap ;
   TopOfHeap := ADR(Store[MaxHeap])
END Init ;


END SysStorage.
